"""
Script to create a project in LabKey.

This script accepts a project name as a command-line argument and creates 
a corresponding project in LabKey. If the project already exists, it notifies the user.

Usage:
    python create_project.py <project_name>
"""

import os
import argparse
from labkey.api_wrapper import APIWrapper
import urllib.parse
import urllib3
from labkey.exceptions import ServerContextError

# Argument parser setup
parser = argparse.ArgumentParser(description='Create a project in LabKey.')
parser.add_argument('project', help='Name of the project to be created')
args = parser.parse_args()
project = args.project

# Connection parameters
labkey_server = "labkey-pro-dev.scicore.unibas.ch"
context_path = ''
api = APIWrapper(labkey_server, '', context_path, use_ssl=True)

def create_container(parent_path, container_name, folder_type):
    """
    Creates a container (project/folder) in LabKey.

    Args:
        parent_path (str): The path of the parent container (empty for root-level projects).
        container_name (str): The name of the container to create.
        folder_type (str): The type of container (e.g., 'Collaboration').

    Returns:
        dict: Response from the API, or an empty dictionary in case of failure.
    """
    create_url = api.server_context.build_url("core", "createContainer.api", container_path=urllib.parse.quote(parent_path))
    create_params = {
        "name": container_name,
        "title": container_name,
        "description": '',
        "folderType": folder_type,
        "isWorkbook": False
    }
    headers = urllib3.util.make_headers()

    try:
        response = api.server_context.make_request(create_url, json=create_params, headers=headers)
        return response
    except ServerContextError as e:
        error_message = str(e)
        if "already exists" in error_message:
            print(f'Container "{container_name}" already exists under "{parent_path}". Continuing...')
            return {"id": "existing_container"}
        else:
            print(f'Error creating container "{container_name}": {error_message}')
            return {}

# Create the project
project_response = create_container('', project, 'Collaboration')

if 'id' in project_response:
    print(f'Successfully created or found existing project "{project}".')
else:
    print(f'Failed to create project "{project}". Response: {project_response}')
