"""
This script connects to a LabKey server using credentials stored in a .netrc file.
It retrieves a list of containers (projects/folders) and extracts their names.
"""

from labkey.api_wrapper import APIWrapper
import pandas as pd
import urllib3
import urllib
import os
import json

# Get credentials from the .netrc file. 
# The .netrc file should be configured with LabKey login details.
# Documentation: https://www.labkey.org/Documentation/wiki-page.view?name=netrc
netrc_file = os.path.join(os.path.expanduser('~'), '.netrc')

# Check if the .netrc file exists
if not os.path.isfile(netrc_file):
    print(f'File does not exist: {netrc_file}')
    exit()

# Read the .netrc file and extract login credentials
# The .netrc file is assumed to contain keys in the format: "machine <hostname> login <username> password <password>"
netrc_df = pd.read_csv(netrc_file, sep=' ', header=None, names=['key', 'value'])

# Ensure required credentials are present
if 'login' not in netrc_df['key'].tolist() or 'password' not in netrc_df['key'].tolist():
    print(f'login and password are required in {netrc_file}')
    exit()

# Extract login and password values
login = netrc_df.loc[netrc_df['key'] == 'login', 'value'].iloc[0]
password = netrc_df.loc[netrc_df['key'] == 'password', 'value'].iloc[0]

# Create authentication headers using Basic Authentication
headers = urllib3.util.make_headers(basic_auth=f'{login}:{password}')

# LabKey server connection parameters
labkey_server = "labkey-pro-dev.scicore.unibas.ch"
container_path = ""  # Use an empty string for the root container
context_path = ''  # For the main LabKey server, use 'labkey'; for Dev, leave it blank

# Initialize the API wrapper to communicate with the LabKey server
api = APIWrapper(labkey_server, context_path, use_ssl=True)

# Define parameters for the API request to fetch containers
params = {
    "includeSubfolders": True,  # Retrieve subfolders recursively
    "depth": 10,  # Maximum depth to search for containers
    "includeEffectivePermissions": False,  # Exclude permission details
    "includeStandardProperties": False  # Exclude additional metadata
}

# Construct the API URL for retrieving container information
url = api.server_context.build_url("project", "getContainers.view", container_path=container_path)

# Send a request to the LabKey server to retrieve container data
resp = api.server_context.make_request(
    url, urllib.parse.urlencode(params).encode(), headers=headers, non_json_response=False
)

# Uncomment the line below to inspect the response structure
# print(json.dumps(resp, indent=4))

def extract_names(data):
    """
    Recursively extracts all container names from the JSON response.

    Args:
        data (dict): JSON response from the LabKey API.

    Returns:
        list: A list of container names.
    """
    names = []
    
    # Add the current container's name if it exists
    if 'name' in data:
        names.append(data['name'])
    
    # Recursively process child containers if they exist
    if 'children' in data and isinstance(data['children'], list):
        for child in data['children']:
            names.extend(extract_names(child))  # Recursive call
    
    return names

# Extract all container names from the API response
all_names = extract_names(resp)

# Print the list of container names
print(all_names)